
% WHAT IF...? MIDDLE LAYER

% FIG 4A row 1

% NOTE!!!!! The values of M and terminals will change if the terminal to gr 
% cell ratio changes

M = 3645; % Assumes a terminal to gr cell divergence ratio of 1:50

terminals = 700; % the total number of terminals per field, derived from 
% 8750 gr cells per field and mf terminal to gr cell divergence of 1:50

mf = 182; % The number of mossy fibres that innervate a field, an estimate 
% derived in the mf stats code paper

ranks = 41; % The number of ranks that supply parallel fibre input to a 
% microzone in the middle
files = 100; % The number of files that cross ranks at right angles

% (1) GENERATE MOSSY FIBRE RATES RECEIVED BY EACH RANK
% Randomly generates values that represent mossy fibre rates received as 
% input to a rank. The number of rates is equal to the number of mossy
% fibres which innervate a rank.  

% % UNIFORM RANDOM DISTRIBUTION
% % The physiological range of mf firing rates in the behaving animal is 
% % 50-300 Hz. The region providing pf input to a microzone can be thought of
% % as divided functionally into 41 ranks crossed by 100 files
% 
% rankrates = zeros(M,ranks); % preallocates storage for an M x ranks matrix
% murank = zeros(1,ranks);
% sigrank = zeros(1,ranks);
% for i = 1:ranks
%     rankrates(:,i) = randi([50 300],1,M)'; % each iteration of the loop 
%     % creates a column vector of M integers randomly generated in the range
%     % 50-300, representing mossy fibre rates received as input to a rank,
%     % with a uniform distribution (i.e. all values are equally likely)
%     murank = mean(rankrates(:,i));
%     sigrank = std(rankrates(:,i));
% end

% % RANDOMLY SAMPLED FROM A NORMAL DISTRIBUTION WITH SPECIFIED MEAN AND SD
% m = 100;
% sd = 20;
% % sd = 40;
% % sd = 50;
% 
% for i = 1:ranks
%     rankrates(:,i) = sd.*randn(1,M) + m; % array of M integers randomly 
%     % drawn from a normal distribution with mean m and standard deviation 
%     % sd
%     murank = mean(rankrates(:,i));
%     sigrank = std(rankrates(:,i));
% end

% RANDOMLY SAMPLED FROM A NORMAL DISTRIBUTION WITH SPECIFIED BUT VARIABLE 
% MEAN (AND FIXED SD)
m = 175;
sd = 12.5;

for i = 1:ranks
    rankrates(:,i) = sd.*randn(1,M) + m; % array of M integers randomly 
    % drawn from a normal distribution with mean m and standard deviation 
    % sd
    murank = mean(rankrates(:,i));
    sigrank = std(rankrates(:,i));
    if i < 11
        m = m + 10;
    elseif i < 31
        m = m - 10;
    else
        m = m + 10;
    end
end

% (2) SAMPLE MF RATES RECEIVED BY A RANK TO DERIVE RATES RECEIVED BY EACH
% FIELD

fieldrates = cell(files,ranks); % preallocates space for a files x ranks cell 
% matrix
for i = 1:files
    for j = 1:ranks
        sample = datasample(rankrates(:,j),mf);
        fieldrates{i,j} = sample; % each iteration of the loop populates a 
        % cell with mf rates randomly sampled from column j of rankrates
    end
end

% (3) ADD COPIES TO RATES RECEIVED BY EACH FIELD

% mfs contribute a variable number of terminals to a field (each field is 
% represented by a cell in fieldrates).

for i = 1:files
    for j = 1:ranks
        a = [];
        hz = fieldrates{i,j};
        for k = 1:length(hz)
            factor = randi(7); % randomly generated number in the range 1-7
            b = hz(k).*ones(1,factor); % makes copies of each value in the 
            % cell at fieldrates{i,j}. The number of copies is equal to
            % factor
            a = [a b];
        end
        fieldrates{i,j} = a;
    end
end

% (4) PLOT THE MEAN AND SD OF MF RATES IN EACH RANK, AND A HISTOGRAM OF MF 
% RATES IN A FILE SELECTED AT RANDOM

opengl software
figure;

subplot(1,5,1)
M = []; S = [];

for j = 1:41
    R = []; 
    for i = 1:100
        R = [R,fieldrates{i,j}];
    end
    M = [M,mean(R)];
    S = [S,std(R)];
end

x = 1:41;

plot(x,M,'color',[.4 0 0],'LineWidth',1); 
xlabel('ranks'); ylabel('Hz'); title('mf rates: all ranks'); 
xlim([1 41]); ylim([50 300]);
set(gca,'FontSize',11)
hold on
s = patch([x fliplr(x)], [M-S  fliplr(M+S)], [1 .9 .9],'EdgeColor',[.8 .6 .6]);
alpha(s,.3)

subplot(1,5,2)

% Put mf rates in each file into a single cell 

mfrates = cell(files,1); % creates a 100 x 1 empty cell array

for i = 1:files % for each row...
    c = [];
    for j = 1:ranks % ...and within each row, each column...
        d = fieldrates{i,j};
        c = [c d];
    end
    mfrates{i,1} = c; % populates the cell array. 
end

edges = [0 10:10:340 350];
centres = edges(1:end-1) + diff(edges)/2;

histogram(mfrates{randi(100)},edges,...
    'facecolor',[1 .9 .9],'facealpha',.6,'edgecolor',[.8 .6 .6])
xlabel('Hz'); ylabel(''); title('mf rates: a file'); xlim([0 350]);
set(gca,'FontSize',11)

% (5) SAMPLE MF RATES IN EACH FIELD TO REPRESENT INPUT TO GR CELLS. TAKE 
% THE MEAN OF EACH SAMPLE AND THE TOP SLICE FOR EACH FIELD

grcellmeans = cell(files,ranks); % Preallocates space
activegrcellmeans = cell(files,ranks);
for i = 1:files
    for j = 1:ranks
        samplemeans = zeros(1,8750); % Preallocates space
        for k = 1:8750
            samplemeans(k) = mean(datasample(fieldrates{i,j},4)); 
        end
        grcellmeans{i,j} = samplemeans; % assigns the sample means to a 
        % cell in grcellmeans. Each cell represents a field. This data
        % isn't graphed but stored away just in case
        order = sort(samplemeans); % puts samplemeans in ascending order
        toporder = order(end-43:end); % 1 x 44 array, the top slice of the
        % gr cell sample means
        activegrcellmeans{i,j} = toporder; % assigns the top slice to a 
        % cell in activegrcellmeans. Each cell represents a field. 
    end
    i
end

% (6) PUT GR CELL RATES IN A ROW OF FIELDS INTO A SINGLE CELL TO REPRESENT
% PF RATES 

pfrates = cell(files,1); % creates a 100 x 1 empty cell array

for i = 1:files % for each row...
    e = [];
    for j = 1:ranks % ...and within each row, each column...
        f = activegrcellmeans{i,j};
        e = [e f];
    end
    pfrates{i,1} = e; % populates the cell array. Each cell contains a 1 x 
    % 1,804 array. (1,804 is derived from 44 sample means per field x 41
    % columns.)
end

% (7) PLOT THE FREQUENCY DISTRIBUTION OF PF RATES IN A FILE SELECTED AT 
% RANDOM

subplot(1,5,3)
edges = [0 10:10:340 350];
centres = edges(1:end-1) + diff(edges)/2;

histogram(pfrates{randi(100)},edges,...
    'facecolor',[.4 .4 .6],'facealpha',.9)%,'edgecolor',[.2 .4 .2])
xlabel('common scale'); ylabel(''); title('pf rates: a file'); xlim([0 350]);
set(gca,'FontSize',11)

% (8) DERIVE THE NUMBER AND FIRING RATE OF ACTIVE INPUTS TO 16 STELLATE
% CELLS IN EACH FIELD OF THE CENTRE RANK, FIELD BY FIELD

SCinputrates = cell(files,16); % files x 16 empty cell array
SCmeans = zeros(files,16);
for i = 1:files
    for j = 1:16
        TB = inputs(); % a function that returns a number between 1 and 
        % 12 representing the number of active inputs to a stellate
        % cell
        if TB == 0
            sample = 0;
        else
            sample = datasample(pfrates{i,1},TB);
        end        
        SCinputrates{i,j} = sample; % each iteration of the j  
        % loop populates a cell with pf rates received by a stellate cell; 
        % there are 16 samples because there are 16 stellate cells per
        % PC; sample size is randomly variable with the Table B
        % probabilities; the sampled population is pfrates{i,1}.

        muSC = mean(sample); % The mean rate of input to a stellate cell...
        SCmeans(i,j) = muSC; %...is assigned to index position i,j in 
        % the 100 x 16 matrix SCmeans 
    end
end

% (9) PLOT INPUT RATES TO SCs IN THE CENTRE RANK

% Convert SCinputrates - i.e., input rates to 16 SCs per field in the 
% centre rank - to a row vector
step1 = cell(files,1); % creates a 100 x 1 empty cell array

for i = 1:files % for each row...
    g = [];
    for j = 1:16 % ...and within each row, each column...
        h = SCinputrates{i,j};
        g = [g h]; % ...unpack the data in a row of cells to create a 
        % vector array...
    end
    step1{i,1} = g; % ...and repack it in a single cell, creating a single 
    % column of cells, each containing all the input rates to SCs in the 
    % centre field. The data in each cell is a row vector but they are
    % different lengths, hence at this point still stored as a cell array
end

aa = [];
for i = 1:files
    bb = step1{i,1};
    aa = [aa bb];
end

array1 = aa;

subplot(1,5,4)
x = 1:length(array1);
plot(x,array1,'LineStyle','none','Marker','o','MarkerSize',2,'color',[0 0 .4],...
    'MarkerFaceColor',[.62 .74 .83]);%,'LineWidth',.5); 
xlabel('signals'); ylabel('common scale'); title('SCs: home'); ylim([50 300]);
set(gca,'FontSize',11)

% (10) PLOT THE MEAN OF INPUT RATES RECEIVED BY STELLATE CELLS IN THE 
% CENTRE RANK

% Convert SCmeans to a row vector
array2 = SCmeans'; array2 = array2(:)';

subplot(1,5,5)
x = 1:length(array2);
plot(x,array2,'LineStyle','none','Marker','o','MarkerSize',2,'color',[.42 .54 .63],...
    'MarkerFaceColor',[.62 .74 .83]);%,'LineWidth',.5); 
xlabel('SCs'); ylabel(''); title('SCs: home'); ylim([50 300]);
set(gca,'FontSize',11)
